@@ -336,6 +336,27 @@ def make_dataobject_from_ob(rec):
     return lt.DataObject(**dict((k, getattr(rec, k)) for k in lt._object_attrnames(rec)))
 
 
+class TableTypeTests(unittest.TestCase):
+    def test_types(self):
+        from collections.abc import (Callable, Container, Iterable, Collection, Mapping, Reversible, Sequence, Sized)
+
+        tbl = lt.Table()
+        tbl.create_index("idx")
+
+        for superclass in (Callable, Sized, Iterable, Container, Collection, Reversible, Sequence):
+            with self.subTest(superclass=superclass):
+                print(superclass.__name__)
+                self.assertTrue(isinstance(tbl, superclass))
+
+        with self.subTest():
+            print("isinstance(_ObjIndex, Mapping)")
+            self.assertTrue(isinstance(tbl._indexes["idx"], Mapping))
+
+        with self.subTest():
+            print("isinstance(_ObjIndexWrapper, Mapping)")
+            self.assertTrue(isinstance(tbl.by.idx, Mapping))
+
+
 @make_test_classes
 class TableCreateTests:
     def test_inserts(self):
@@ -1125,7 +1146,7 @@ class TableListTests:
         expected = lt.Table().csv_import(textwrap.dedent("""\
         name,mean,min,max,variance,std_dev,count,missing
         a,3.0,1,5,4,2.0,3,0
-        b,3.0,2,4,2,1.4142135623730951,2,1
+        b,3.0,2,4,2,1.414,2,1
         """), transforms={}.fromkeys("mean min max variance std_dev count missing".split(), ast.literal_eval))
         expected.present()
         print(expected.info())
@@ -1842,6 +1863,24 @@ class TableImportExportTests:
             with self.subTest():
                 self.assertEqual(1, len(outlines))
 
+    def test_csv_export_to_string(self):
+        from itertools import permutations
+        test_size = 3
+        t1 = make_test_table(self.make_data_object, test_size)
+        for fieldnames in permutations(list('abc')):
+            out_string = t1.csv_export(None, fieldnames)
+            outlines = out_string.splitlines()
+            with self.subTest():
+                self.assertEqual(','.join(fieldnames), outlines[0])
+            with self.subTest():
+                self.assertEqual(test_size ** 3 + 1, len(outlines))
+            for ob, line in zip(t1, outlines[1:]):
+                csv_vals = line.split(',')
+                with self.subTest(ob=ob, csv_vals=csv_vals):
+                    self.assertTrue(all(
+                        int(csv_vals[i]) == getattr(ob, fld) for i, fld in enumerate(fieldnames)
+                    ))
+
     def test_csv_import(self):
         data = csv_data
         incsv = io.StringIO(data)
@@ -1889,7 +1928,7 @@ class TableImportExportTests:
 
         # test separately, no transforms for JSON imports
         import_source_name = "test/abc.json.gz"
-        tt2 = lt.Table().json_import("test/abc.json.gz")
+        tt2 = lt.Table().json_import("test/abc.json.gz", streaming=True)
         print("abc.json.gz", tt2.info())
         expected_info = {**tt.info(), "name": import_source_name}
         with self.subTest():
@@ -2242,16 +2281,13 @@ class TableImportExportTests:
             tbl.present()
             self.assertEqual([1000, 3.14, None, 'ⓠ*bert'], list(tbl.all.value))
 
-    def test_json_export(self):
+    def test_json_export_streaming(self):
         from itertools import permutations
         test_size = 3
         t1 = make_test_table(self.make_data_object, test_size)
         for fieldnames in permutations(list('abc')):
-            out = io.StringIO()
-            t1.json_export(out, fieldnames)
-            out.seek(0)
-            outlines = out.read().splitlines()
-            out.close()
+            out_string = t1.json_export(None, fieldnames=fieldnames, streaming=True)
+            outlines = out_string.splitlines()
 
             with self.subTest(fieldnames=fieldnames):
                 self.assertEqual(test_size**3, len(outlines))
@@ -2262,10 +2298,27 @@ class TableImportExportTests:
                 with self.subTest(ob=ob, line=line):
                     self.assertEqual(t1_dataobj, lt.DataObject(**json_dict))
 
+    def test_json_export_nonstreaming(self):
+        from itertools import permutations
+        import json
+        test_size = 3
+        t1 = make_test_table(self.make_data_object, test_size)
+        for fieldnames in permutations(list('abc')):
+            out_string = t1.json_export(None, fieldnames=fieldnames, streaming=False)
+            observed_json = json.loads(out_string)
+
+            with self.subTest(fieldnames=fieldnames):
+                self.assertEqual(test_size**3, len(observed_json))
+
+            for ob, json_dict in zip(t1, observed_json):
+                t1_dataobj = make_dataobject_from_ob(ob)
+                with self.subTest(ob=ob, json_dict=json_dict):
+                    self.assertEqual(t1_dataobj, lt.DataObject(**json_dict))
+
     def test_json_import(self):
         data = json_data
         injson = io.StringIO(data)
-        jsontable = lt.Table().json_import(injson, transforms={'a': int, 'b': int, 'c': int})
+        jsontable = lt.Table().json_import(injson, streaming=True, transforms={'a': int, 'b': int, 'c': int})
 
         test_size = 3
         t1 = make_test_table(self.make_data_object, test_size)
@@ -2277,7 +2330,7 @@ class TableImportExportTests:
 
     def test_json_string_import(self):
         data = json_data
-        jsontable = lt.Table().json_import(data, transforms={'a': int, 'b': int, 'c': int})
+        jsontable = lt.Table().json_import(data, streaming=True, transforms={'a': int, 'b': int, 'c': int})
 
         test_size = 3
         t1 = make_test_table(self.make_data_object, test_size)
@@ -2289,7 +2342,7 @@ class TableImportExportTests:
 
     def test_json_string_list_import(self):
         data = json_data
-        jsontable = lt.Table().json_import(data.splitlines(), transforms={'a': int, 'b': int, 'c': int})
+        jsontable = lt.Table().json_import(data.splitlines(), streaming=True, transforms={'a': int, 'b': int, 'c': int})
 
         test_size = 3
         t1 = make_test_table(self.make_data_object, test_size)
@@ -2299,6 +2352,30 @@ class TableImportExportTests:
         with self.subTest():
             self.assertEqual(len([d for d in data.splitlines() if d.strip()]), len(jsontable))
 
+    def test_json_nonstreaming_with_path_import(self):
+        data = json_data
+        data = ',\n'.join(data.rstrip().splitlines())
+        json_input0 = '[' + data + ']'
+        json_input1 = '{ "data": [' + data + ']}'
+        json_input2 = '{ "data": { "items": [' + data + ']}}'
+
+        for json_input, path in [
+            (json_input0, ""),
+            (json_input1, "data"),
+            (json_input2, "data.items"),
+        ]:
+            jsontable = lt.Table().json_import(json_input,
+                                               path=path,
+                                               transforms={'a': int, 'b': int, 'c': int})
+
+            test_size = 3
+            t1 = make_test_table(self.make_data_object, test_size)
+
+            with self.subTest():
+                self.assertTrue(all(make_dataobject_from_ob(rec1) == rec2 for rec1, rec2 in zip(t1, jsontable)))
+            with self.subTest():
+                self.assertEqual(len([d for d in data.splitlines() if d.strip()]), len(jsontable))
+
     def test_fixed_width_import(self):
         data = fixed_width_data
         data_file = io.StringIO(data)
